/*

Copyright (c) 2006, Eric Tiedemann
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:


* Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

      * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.

      * Neither the name of Eric Tiedemann nor the names of his
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef E7_RATIONAL_H
#define E7_RATIONAL_H

#include "bignum.h"

namespace e7 {

  class Rational {

  public:

    Bignum num, den;

    Rational(Bignum num, Bignum den);
    Rational(int64_t num, int64_t den);
    Rational(Bignum num);
    Rational(int64_t num);

    ~Rational();

    void canonicalize();

    bool integralp();

    static bool cmp(Rational a, Rational b);

    Rational abs() { return Rational(num.abs(), den); }
    Rational negative() { return Rational(num.negative(), den); }

    bool zerop() { return num.zerop(); }
    bool positivep() { return num.positivep(); }
    bool negativep() { return num.negativep(); }

    friend const  Rational operator+(Rational, Rational);
    friend const  Rational operator-(Rational, Rational);
    friend const  Rational operator*(Rational, Rational);
    friend const  Rational operator/(Rational, Rational);
    
    friend bool operator==(Rational, Rational);
    friend bool operator<(Rational, Rational);
    friend bool operator>(Rational, Rational);
    friend bool operator<=(Rational, Rational);
    friend bool operator>=(Rational, Rational);
    friend bool operator!=(Rational, Rational);
    
  };

  extern Rational expt(Rational x, Bignum n);

  
  std::ostream &operator<<(std::ostream &out, Rational r);
  
}

#endif
