/*

Copyright (c) 2006, Eric Tiedemann
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:


* Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

      * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.

      * Neither the name of Eric Tiedemann nor the names of his
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <iostream>

#include <stdio.h>
#include <string.h>


#include "bignum.h"
#include "rational.h"


// debug print tools
#define display(expr) std::cerr << #expr << " = " << (expr) << std::endl;
#define display2(expr1, expr2) display(expr1); display(expr2);
#define display3(expr1, expr2, expr3) display(expr1); display(expr2); display(expr3);


using namespace e7;
using namespace e7::scheme48;


int main(int argc, char **argv) {

  e7::Bignum::init();

  Bignum x(argv[1], 10, 0);
  Bignum y(argv[2], 10, 0);

  display(gcd(x, y));

  display(remainder(x, y));

  display(quotient(x, y));

  int rc = s48_bignum_fits_in_word_p(x.rep->b, 64, 1);

  display(rc);

  Bignum zz(s48_bignum_length_upper_limit());

  Bignum qq = y * y * y * y * y * y * y * y * y * y * y;

  display(x * y);

  display(x - y);

  display2(x, y);

  display(x + y);

  display(qq);

  display(s48_bignum_fits_in_word_p(qq.rep->b, 64, 1));

  //display(zz);

  display(s48_bignum_max_digit_stream_radix());

  display(s48_bignum_length_upper_limit());

  display(BIGNUM_DIGIT_LENGTH );
  display(BIGNUM_HALF_DIGIT_LENGTH );
  display(BIGNUM_RADIX );
  display(BIGNUM_RADIX_ROOT );
  display(BIGNUM_DIGIT_MASK	 );
  display(BIGNUM_HALF_DIGIT_MASK	 );

  display(BIGNUM_DIGITS_FOR_LONG);


  display(Rational(4, 3) * Rational(4, 3));

  display(expt(Rational(9, 8), Bignum(int64_t(5))));

  display(expt(Bignum(int64_t(3)), Bignum(int64_t(5))));

  return 0;
}
